;;; $Id: alloc.scm,v 3.9 1994/03/06 20:47:49 queinnec Exp $
;;; Copyright (c) 1990-93 by Christian Queinnec. All rights reserved.

;;;                ***********************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon 
;;;                              Christian Queinnec  
;;;                   \'Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;; This file defines the basic machinery to allocate instances. It is
;;; not general since it is not yet possible to define generic
;;; functions. This will come later with maker.scm to implement
;;; general-make.  A bunch of makers are precomputed, it is possible
;;; to augment the size of the bunch if you care about it.

;;; Try to create appropriate makers with a correct arity to avoid
;;; using general-make and consing a lot.

(define (make-maker class)
  (let ((poly-fields-number (count-Poly-Fields (careless-Class-fields class))))
    (case poly-fields-number
      ((0) (create-zero-poly-maker class))
      ((1) (if (Poly-Field? (car (last-pair (careless-Class-fields class))))
               (create-one-final-poly-maker class)
               (lambda args (general-make class args)) ))
      (else (lambda args (general-make class args))) ) ) )

;;;oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
;;; Precompute a bunch of makers so they can be shared.

;;; Makers without Poly-Field.

(define (generate-zero-poly-maker class-variable field-names)
  `(lambda ,field-names
     (initialize! (instance (careless-Class-number ,class-variable)
                            ,@field-names )) ) )

(generate-bunch-of-zero-poly-makers 10)

;;; Same trick for makers with only one final Poly-Field.

(define (generate-one-final-poly-maker class-variable field-names)
  (let* ((temp (reverse field-names))
         (poly (car temp))
         (regular (reverse! (cdr temp)))
         (regular-length (length regular))
         (g (gensym))
         (l (gensym)) )
    `(lambda ,(let hook ((items regular))
                (if (pair? items)
                    (cons (car items) (hook (cdr items)))
                    (cons l poly) ) )
       (initialize!
        (let* ((,g (length ,poly))
               (v (allocate-empty-instance 
                   (careless-Class-number ,class-variable)
                   (+ ,regular-length 1 ,g) )) )
          (check-appropriate-size ,g ,l v)
          ,@(oo-map (lambda (fn i) `(instance-set! v ,i ,fn))
                    regular
                    (iota 0 regular-length) )
          (instance-set! v ,regular-length ,l)
          (fill-instance! v ,(+ regular-length 1) ,poly) ) ) ) ) )
                                           
(generate-bunch-of-one-final-poly-makers 10)

;;; end of alloc.scm
