;;; $Id: check.scm,v 3.1 1994/02/06 17:07:35 queinnec Exp $

;;; This file inspects the class hierarchy to find inconsistencies. It
;;; is needed when I debug, not in the distribution. To avoid problems
;;; it does not use generic functions.

;;; This variable should be set to #T if one wants to fix the detected
;;; inconsistencies. One should of course correct the source code !

(define *fix-inconsistencies* #f)

;;; Notify the inconsistency.

(define (check-error msg . culprits)
  (display "***** Inconsistency on: ")
  (display msg)
  (for-each (let ((i -1))
              (lambda (v) 
                (set! i (+ 1 i))
                (newline)
                (display "[")(display i)(display "]-> ")
                (display v) ) )
            culprits )
  (newline)
  ;; Raise a real anomaly if in debug mode.
  (when *meroon-debug*
    (oo-error 'check-class-consistency "~A: ~A" msg culprits) ) )

;;; Check the consistency of a class. One should also check min- and
;;; max-son numbers.

(define (check-class-consistency c)
  (let ((ncn (Class-number c))
        (depth (Class-super-length c)) )
    (unless (<= ncn *class-number*)
      (check-error "Too big class number" c) )
    (for-each check-field-consistency (Class-fields c))
    ;; direct super number is not up to date!
    (when (Class-super-number c) 
      (unless (= (Class-super-number c) 
                 (current-class-number (Class-super-number c)) )
        (when *fix-inconsistencies*
          (set-Class-super-number!
           c (current-class-number (Class-super-number c)) ) )
        (check-error "Inconsistent class super number" c) ) )
    ;; one of the super numbers is out of date
    (do ((i 0 (+ 1 i)))
        ((= i depth))
      (unless (= (Class-super c i)
                 (current-class-number (Class-super c i)) )
        (when *fix-inconsistencies*
          (set-Class-super! c i (current-class-number (Class-super c i))) )
        (check-error "Inconsistent super number" c i) ) )
    ;; recurse on subclasses
    (for-each (lambda (cn) (check-class-consistency (number->class cn)))
              (Class-subclass-numbers c) ) ) )

(define (check-field-consistency field)
  (let ((cn (Field-class-number field)))
    (unless (= cn (current-class-number cn))
      (when *fix-inconsistencies*
        (set-Field-class-number! field (current-class-number cn)) )
      (check-error "Inconsistent field" field) )
    #t ) )

;;; Check the whole hierarchy with:
;;;               (check-class-consistency Object-class)

;;; end of check.scm
