;;; $Id: clone.scm,v 3.9 1994/09/28 16:40:14 queinnec Exp $
;;; Copyright (c) 1990-93 by Christian Queinnec. All rights reserved.

;;;                ************************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon 
;;;                              Christian Queinnec  
;;;                   \'Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;; This file defines the (duplicate (<instance> class-name) :kw ... :kw
;;; ...) special form.  This form creates a new instance with explicit
;;; fields stated as in the instantiate form by keyords and other
;;; fields taken from <instance>.

;;; Parse the duplicate form. 

(define (process-duplicate-form desc parms)
  (unless (and (pair? desc)
               (symbol? (car desc))
               (pair? (cdr desc))
               (symbol? (cadr desc))
               (null? (cddr desc)) )
    (report-meroon-error
     'Syntax 'duplicate "Incorrect specification" desc ) )
  (process-duplication
   (car desc)
   (symbol->class (cadr desc)
                  (lambda (name)
                    (report-meroon-error
                     'Syntax 'duplicate "No such class" name ) ) )
   parms ) )

;;; The process-duplication function mimics the process-instantiation form.

(define (process-duplication ins-tance class parms)
  (let* ((fields (careless-Class-fields class))
         (class-name (symbol-concatenate (careless-Class-name class) '-class))
         (g (gensym))
         (org (gensym))
         (index (gensym))
         (contents (arrange-whole-initialization 
                    org
                    fields
                    (process-initialization fields parms find-duplication) )) )
    `(let ((,index 0)
           (,org ,ins-tance))
       (if (is-a? ,org ,class-name)
           (let ((,g ((careless-Class-allocator ,class-name)
                      ,@(generate-whole-size fields contents) )))
             ,@(generate-whole-initialization g index fields contents)
             (initialize! (fill-other-fields-from-instance! 
                           ,g ,org )) )
           (report-meroon-error
            'Syntax 'duplicate
            "Original object has not the proper class"
            ,org ,class-name ) ) ) ) )

;;; This set of functions take a description of the content of an
;;; instance to be allocated and guesses the missing components based
;;; on the instance which is to be duplicated.

(define (arrange-whole-initialization org fields contents)
  (oo-map (lambda (field content)
            (if (pair? content)
                content
                ;; This means take the content of the original instance
                (arrange-initialization org field content) ) )
          fields
          contents ) )

(define-generic (arrange-initialization org (field Field) content))

(define-method (arrange-initialization org (field Mono-Field) content)
  (list 1 'meroon-uninitialized) )

(define-method (arrange-initialization org (field Poly-Field) content)
  (let* ((fname (Field-name field)))
    (list `(field-length ,org ',fname)) ) )

;;; This generic function parses the parameters of the duplicate form
;;; and returns a content specification.

(define-generic (find-duplication (f Field) parms k))

(define-method (find-duplication (f Mono-Field) parms k)
  (let* ((form (find-option-single-value 
                (careless-Field-name f) parms option-not-there ))
         (other-parms (remove-option (careless-Field-name f) parms)) )
    (if (absent-option? form)
        ;; means that the real value will be copied from the original instance
        (k (list) other-parms)
        (k (list 1 form) other-parms) ) ) )

;;; This function does not recognize the :kw-length specification.

(define-method (find-duplication (f Poly-Field) parms k)
  ;; check if the field is specified by comprehension
  (let* ((content (find-option-values
                   (careless-Field-name f) parms option-not-there ))
         (parms (remove-option (careless-Field-name f) parms)) )
    (if (absent-option? content)
        ;; means that the size should be copied from the original instance
        (k (list) parms)
        (k (cons (length content) content) parms) ) ) )

;;; Fill uninitialized fields of O from ORIGINAL.

(define (fill-other-fields-from-instance! o original)
  (for-each 
   (lambda (field) (fill-uninitialized-field-from-instance! o field original))
   (careless-Class-fields (object->class o)) )
  o )

(define-generic (fill-uninitialized-field-from-instance! o (field) original))

(define-method (fill-uninitialized-field-from-instance! 
                o (field Mono-Field) original )
  (let ((offset (compute-value-offset o field)))
    (when (uninitialized? (instance-ref o offset))
      (let ((orgoffset (compute-value-offset original field)))
        (instance-set! o offset (instance-ref original orgoffset)) ) ) ) )

;;; Only the first fields of the duplicated instance are copied from
;;; the original. Should share better offsets computations    FUTURE

(define-method (fill-uninitialized-field-from-instance! 
                o (field Poly-Field) original )
  (let ((len (min (field-length o field)
                  (field-length original field) )))
    (do ((i 0 (+ i 1)))
        ((>= i len) o)
      (unless (field-defined? o field i)
        (let ((offset (compute-value-offset o field i))
              (orgoffset (compute-value-offset original field i)) )
          (instance-set! o offset (instance-ref original orgoffset)) ) ) ) ) )
  
;;; end of clone.scm
