;;; $Id: instance.scm,v 3.9 1994/02/19 19:47:16 queinnec Exp $
;;; Copyright (c) 1990-93 by Christian Queinnec. All rights reserved.

;;;                ***********************************************
;;;                    Small, Efficient and Innovative Class System
;;;                                    Meroon V3
;;;                              Christian Queinnec  
;;;                   \'Ecole Polytechnique & INRIA--Rocquencourt
;;;                ************************************************

;;; This file defines the representation of instances. They are
;;; implemented as vectors with a first slot holding the instantiation
;;; link. The words offset and index are distinguished; the former
;;; denotes offsets within the instance representation while the
;;; latter denotes natural numbers used within indexed fields.

;;; This file only contains functions. The associated macros are in the
;;; macinst.scm file.

;;; Test if a Scheme object is a Meroon object. 
;;; Rather weak ! Could be improved when Scheme can create new types.
;;; Note that native Scheme objects are not Meroon objects and no attempt 
;;; is done to handle that since this would pose many problems if the user
;;; wants, for example, to subclass vectors or functions.

(define (Object? o)
  (and (vector? o)
       (number? (object->class-number o)) ) )

;;; Extracts the internal number of a class from within an instance.

(define (object->class-number o)
  (vector-ref o 0) )

;;; Classes are sometimes renumbered, this function updates the
;;; class-number of an instance. Return #t if the class-number was
;;; really obsolete.

(define (update-object->class-number! o)
  (let* ((ocn (object->class-number o))
         (ncn (current-class-number ocn)) )
    (vector-set! o 0 ncn)
    (not (= ocn ncn)) ) )

;;; Shallow copy of an instance.

(define (instance-clone o)
  (let* ((n (vector-length o))
         (r (make-vector n)) )
    (copy-vector-slice! o r 0 n)
    r ) )

;;; end of instance.scm
